/*
  ========================================================================

  FILE:                 AEESQLFunc.h
  
  SERVICES:             ISQLFunc
                        ISQLFuncLib

  GENERAL DESCRIPTION:  This header defines an interface to the ISQL
                        database extension that allows applications to
                        register user-defined functions and the interface
                        that a user-defined function library must adhere
                        to.

  ========================================================================
  ========================================================================
    
               Copyright  1999-2007 QUALCOMM Incorporated 
                     All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
    
  ========================================================================
  ========================================================================
*/

#ifndef __AEEISQLFUNC_H__
#define __AEEISQLFUNC_H__

#include "AEESQL.h"

#define AEEIID_SQLFUNC     0x0103f1e8
#define AEEIID_SQLUDFLIB   0x0103f35c

// Forward declarations
typedef struct ISQLFunc    ISQLFunc;
typedef struct ISQLUDFLib  ISQLUDFLib;

//********************************************************************************************************************************
//
// ISQLFunc Interface
//
//********************************************************************************************************************************

// Text representations a user-defined function understands
#define AEESQL_UTF8                 1
#define AEESQL_UTF16LE              2
#define AEESQL_UTF16BE              3
#define AEESQL_UTF16                4    // Use native byte order 
#define AEESQL_ANY                  5    // sqlite3_create_function only
#define AEESQL_UTF16_ALIGNED        8    // sqlite3_create_collation only

// autoload library behavior
#define AEESQLF_REMOVEUDFLIBRARY    0x00000000
#define AEESQLF_LOADUDFLIBRARY      0x00000001
#define AEESQLF_AUTOLOAUDFLIBRARY   0x00000002

// User-defined function parameter structure
typedef struct {
   int eType;                    // data type
   int dwSize;                   // size of data
   int dwFlags;                  // flags, like SQLF_WIDE
   void (*pfnFree)(void *pData); // data dtor, if required
   aeesql_uint64_struct i;       // integer
   const void *p;                // pointer data
} AEESQLFuncParam;

// User-defined function prototypes
typedef int (*PFNSQLUDFFUNC)(void *pUserData, int nArg, AEESQLFuncParam *pArgs, AEESQLFuncParam *pResult);  
typedef int (*PFNSQLUDFSTEP)(void *pUserData, int dwStep, int nArg, AEESQLFuncParam *pArgs, AEESQLFuncParam *pResult);  
typedef int (*PFNSQLUDFFINAL)(void *pUserData, AEESQLFuncParam *pResult);  
typedef int (*PFNSQLUDFCOMPARE)(void *pUserData, int dwLen1, const void *pString1, int dwLen2, const void *pString2);  

// ISQLFunc interface
AEEINTERFACE(ISQLFunc)
{
   INHERIT_IQueryInterface(ISQLFunc);

   int   (*CreateFunction)    (ISQLFunc *piISQLFunc, const void *pFuncName, uint32 flags, int nArgs, int dwTextRep, PFNSQLUDFFUNC pfnFunc, void *pUserData);
   int   (*CreateAggregate)   (ISQLFunc *piISQLFunc, const void *pFuncName, uint32 flags, int nArgs, int dwTextRep, PFNSQLUDFSTEP pfnStep, PFNSQLUDFFINAL pfnFinal, void *pUserData);
   int   (*CreateCollation)   (ISQLFunc *piISQLFunc, const void *pFuncName, uint32 flags, int dwTextRep, PFNSQLUDFCOMPARE pfnCompare, void *pUserData);
   int   (*LoadLibrary)       (ISQLFunc *piISQLFunc, const char *pszMimeType, AEECLSID clsId, int dwLoadFlags);
};

// IQueryInterface Functions
static __inline uint32 ISQLFUNC_AddRef(ISQLFunc *piISQLFunc) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->AddRef(piISQLFunc);
}
static __inline uint32 ISQLFUNC_Release(ISQLFunc *piISQLFunc) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->Release(piISQLFunc);
}
static __inline int ISQLFUNC_QueryInterface(ISQLFunc *piISQLFunc, AEECLSID idReq, void **ppiISQLFunc) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->QueryInterface(piISQLFunc, idReq, ppiISQLFunc);
}

// User-defined function creation
static __inline int ISQLFUNC_CreateFunction(ISQLFunc *piISQLFunc, const char *pFuncName, int nArgs, int dwTextRep, PFNSQLUDFFUNC pfnFunc, void *pUserData) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->CreateFunction(piISQLFunc, (const void *)pFuncName, 0, nArgs, dwTextRep, pfnFunc, pUserData);
}
static __inline int ISQLFUNC_CreateWFunction(ISQLFunc *piISQLFunc, const AECHAR *pawFuncName, int nArgs, int dwTextRep, PFNSQLUDFFUNC pfnFunc, void *pUserData) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->CreateFunction(piISQLFunc,(const void *)pawFuncName, SQLF_WIDE, nArgs, dwTextRep, pfnFunc, pUserData);
}

// User-defined aggregate function creation
static __inline int ISQLFUNC_CreateAggregate(ISQLFunc *piISQLFunc, const char *pFuncName, int nArgs, int dwTextRep, PFNSQLUDFSTEP pfnStep, PFNSQLUDFFINAL pfnFinal, void *pUserData) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->CreateAggregate(piISQLFunc, (const void *)pFuncName, 0, nArgs, dwTextRep, pfnStep, pfnFinal, pUserData);
}
static __inline int ISQLFUNC_CreateWAggregate(ISQLFunc *piISQLFunc, const AECHAR *pawFuncName, int nArgs, int dwTextRep, PFNSQLUDFSTEP pfnStep, PFNSQLUDFFINAL pfnFinal, void *pUserData) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->CreateAggregate(piISQLFunc,(const void *)pawFuncName, SQLF_WIDE, nArgs, dwTextRep, pfnStep, pfnFinal, pUserData);
}

// User-defined collation creation
static __inline int ISQLFUNC_CreateCollation(ISQLFunc *piISQLFunc, const char *pFuncName, int dwTextRep, PFNSQLUDFCOMPARE pfnCompare, void *pUserData) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->CreateCollation(piISQLFunc, (const void *)pFuncName, 0, dwTextRep, pfnCompare, pUserData);
}
static __inline int ISQLFUNC_CreateWCollation(ISQLFunc *piISQLFunc, const AECHAR *pawFuncName, int dwTextRep, PFNSQLUDFCOMPARE pfnCompare, void *pUserData) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->CreateCollation(piISQLFunc,(const void *)pawFuncName, SQLF_WIDE, dwTextRep, pfnCompare, pUserData);
}

// User-defined function library loading
static __inline int ISQLFUNC_LoadLibrary(ISQLFunc *piISQLFunc, const char *pszMimeType, AEECLSID clsId, int dwFlags) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->LoadLibrary(piISQLFunc, pszMimeType, clsId, dwFlags);
}
static __inline int ISQLFUNC_LoadLibraryByMimetype(ISQLFunc *piISQLFunc, const char *pszMimeType) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->LoadLibrary(piISQLFunc, pszMimeType, 0, AEESQLF_LOADUDFLIBRARY);
}
static __inline int ISQLFUNC_LoadLibraryByClassId(ISQLFunc *piISQLFunc, AEECLSID clsId) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->LoadLibrary(piISQLFunc, NULL, clsId, AEESQLF_LOADUDFLIBRARY);
}

// Add a user-defined function library to the autoload table
static __inline int ISQLFUNC_AddAutoLoadLibraryByMimetype(ISQLFunc *piISQLFunc, const char *pszMimeType) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->LoadLibrary(piISQLFunc, pszMimeType, 0, AEESQLF_AUTOLOAUDFLIBRARY);
}
static __inline int ISQLFUNC_AddAutoLoadLibraryByClassId(ISQLFunc *piISQLFunc, AEECLSID clsId) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->LoadLibrary(piISQLFunc, NULL, clsId, AEESQLF_AUTOLOAUDFLIBRARY);
}

// Remove a user-defined function library from the autoload table
static __inline int ISQLFUNC_RemoveAutoLoadLibraryByMimetype(ISQLFunc *piISQLFunc, const char *pszMimeType) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->LoadLibrary(piISQLFunc, pszMimeType, 0, AEESQLF_REMOVEUDFLIBRARY);
}
static __inline int ISQLFUNC_RemoveAutoLoadLibraryByClassId(ISQLFunc *piISQLFunc, AEECLSID clsId) {
   return AEEGETPVTBL(piISQLFunc,ISQLFunc)->LoadLibrary(piISQLFunc, NULL, clsId, AEESQLF_REMOVEUDFLIBRARY);
}

// Get helpers for AEESQLFuncParam
static __inline int GetAEESQLFuncParamInt(AEESQLFuncParam *p, int *pdwValue) {
   if (AEESQL_COL_DT_INTEGER == p->eType) {
      *pdwValue = (int)aeesql_int64struct_to_int64(p->i);;
      return SUCCESS;
   }
   return EFAILED;
}
static __inline int GetAEESQLFuncParamInt64(AEESQLFuncParam *p, int64 *piValue) {
   if (AEESQL_COL_DT_INTEGER == p->eType) {
      *piValue = aeesql_int64struct_to_int64(p->i);
      return SUCCESS;
   }
   return EFAILED;
}
static __inline int GetAEESQLFuncParamFloat(AEESQLFuncParam *p, double *pdValue) {
   if (AEESQL_COL_DT_FLOAT == p->eType) {
      *pdValue = aeesql_doublestruct_to_double(p->i);
      return SUCCESS;
   }
   return EFAILED;
}
static __inline int GetAEESQLFuncParamText(AEESQLFuncParam *p, const char **ppszText, int *dwNumBytes) {
   if (AEESQL_COL_DT_TEXT == p->eType) {
      *ppszText = (char *)p->p;
      *dwNumBytes = p->dwSize;
      return SUCCESS;
   }
   return EFAILED;
}
static __inline int GetAEESQLFuncParamWText(AEESQLFuncParam *p, const AECHAR **ppawText, int *dwNumBytes) {
   if (AEESQL_COL_DT_TEXT == p->eType) {
      *ppawText = (AECHAR *)p->p;
      *dwNumBytes = p->dwSize;
      return SUCCESS;
   }
   return EFAILED;
}
static __inline int GetAEESQLFuncParamBlob(AEESQLFuncParam *p, const void **ppBlob, int *dwNumBytes) {
   if (AEESQL_COL_DT_BLOB == p->eType) {
      *ppBlob = p->p;
      *dwNumBytes = p->dwSize;
      return SUCCESS;
   }
   return EFAILED;
}

// Set helpers for AEESQLFuncParam
static __inline void SetAEESQLFuncParamInt(AEESQLFuncParam *p, int dwValue) {
   p->eType = AEESQL_COL_DT_INTEGER;
   p->i = aeesql_int64struct_from_int64((int64)dwValue);
   p->dwSize = sizeof(int);
   p->dwFlags = 0;
   p->pfnFree = NULL;
}
static __inline void SetAEESQLFuncParamInt64(AEESQLFuncParam *p, int64 diValue) {
   p->eType = AEESQL_COL_DT_INTEGER;
   p->i = aeesql_int64struct_from_int64(diValue);
   p->dwSize = sizeof(int64);
   p->dwFlags = 0;
   p->pfnFree = NULL;
}
static __inline void SetAEESQLFuncParamFloat(AEESQLFuncParam *p, double dValue) {
   p->eType = AEESQL_COL_DT_FLOAT;
   p->i = aeesql_doublestruct_from_double(dValue);
   p->dwSize = sizeof(double);
   p->dwFlags = 0;
   p->pfnFree = NULL;
}
static __inline void SetAEESQLFuncParamText(AEESQLFuncParam *p, const char *pszText, int dwNumBytes, void (*pfnFree)(void *pData)) {
   p->eType = AEESQL_COL_DT_TEXT;
   p->p = (void *)pszText;
   p->dwSize = dwNumBytes;
   p->dwFlags = 0;
   p->pfnFree = pfnFree;
}
static __inline void SetAEESQLFuncParamWText(AEESQLFuncParam *p, const AECHAR *pawText, int dwNumBytes, void (*pfnFree)(void *pData)) {
   p->eType = AEESQL_COL_DT_TEXT;
   p->p = (void *)pawText;
   p->dwSize = dwNumBytes;
   p->dwFlags = SQLF_WIDE;
   p->pfnFree = pfnFree;
}
static __inline void SetAEESQLFuncParamBlob(AEESQLFuncParam *p, const void *pBlob, int dwNumBytes, void (*pfnFree)(void *pData)) {
   p->eType = AEESQL_COL_DT_BLOB;
   p->p = pBlob;
   p->dwSize = dwNumBytes;
   p->dwFlags = 0;
   p->pfnFree = pfnFree;
}
static __inline void SetAEESQLFuncParamNull(AEESQLFuncParam *p) {
   p->eType = AEESQL_COL_DT_NULL;
   p->i = aeesql_int64struct_from_int64((int64)0);
   p->dwSize = 0;
   p->dwFlags = 0;
   p->pfnFree = NULL;
}

//********************************************************************************************************************************
//
// ISQLUDFLib Interface
//
//********************************************************************************************************************************

// ISQLUDFLib mimetype string
#define AEESQL_UDFLIBMIMETYPE "x-qc-sqludflib"

// name of the auto-load table
#define AEESQL_UDFAUTOLOADTABLENAME "BREWUDFLibAutoLoadTable"

// name of the auto-load table column that stores info to find the libraries
#define AEESQL_UDFAUTOLOADTABLECOLNAME "library"

// ISQLUDFLib interface
AEEINTERFACE(ISQLUDFLib)
{
   INHERIT_IQueryInterface(ISQLUDFLib);

   int  (*Init)(ISQLUDFLib *po, ISQLFunc *piSQLFunc);
};

static __inline uint32 ISQLUDFLIB_AddRef(ISQLUDFLib *po) {
   return AEEGETPVTBL(po,ISQLUDFLib)->AddRef(po);
}
static __inline uint32 ISQLUDFLIB_Release(ISQLUDFLib *po) { 
   return AEEGETPVTBL(po,ISQLUDFLib)->Release(po);
}
static __inline int ISQLUDFLIB_QueryInterface(ISQLUDFLib *po, AEECLSID idReq, void **ppo) {
   return AEEGETPVTBL(po,ISQLUDFLib)->QueryInterface(po, idReq, ppo);
}
static __inline int ISQLUDFLIB_Init(ISQLUDFLib *po, ISQLFunc *piSQLFunc) {
   return AEEGETPVTBL(po,ISQLUDFLib)->Init(po, piSQLFunc);
}

/*==============================================================================
  DATA STRUCTURE DOCUMENTATION
================================================================================
ISQLFunc Interface and Class IDs

Description:
   Each class and interface is identified by a unique ID.  These IDs are used to 
   distinguish between various types of objects.

   ISQLFunc is derived from ISQL and can not be created on its own.  Instead
   of calling ISHELL_CreateInstance() with a class id for ISQLFunc,
   ISQL_QueryInterface() is used to retrieve the ISQLFunc interface from an
   existing ISQL object:

===pre>
   nErr = ISQL_QueryInterface(piSQL, AEEIID_SQLFUNC, (void **)piSQLFunc);
===/pre> 

   ISQLFunc is identified by the following interface ID:

Definition:

Identifier           Definition
----------           -----------------------------------------------------------
AEEIID_SQLFUNC:      Identifies the ISQLFunc interface

Comments:
   None

See Also:
   ISHELL_CreateInstance()

================================================================================

User-defined function text representations

Description:
   User-define functions are called from SQL statements.  They are typically
   passed text-based paramters.  The following constants denote what type
   of text arguments a user-defined function understands.

Definition:
===pre>
   Definition                    Description
   ----------------------        -------------------------------------------------------
   #define AEESQL_UTF8           single byte (8 bit) text
   #define AEESQL_UTF16LE        double byte (16 bit) little endian text
   #define AEESQL_UTF16BE        double byte (16 bit) big endian text
   #define AEESQL_UTF16          double byte (16 bit) native byte order 
   #define AEESQL_ANY            ISQLFUNC_CreateFunction() only
   #define AEESQL_UTF16_ALIGNED  ISQLFUNC_CreateCollation() only
===/pre>

Members:
   None
   
Comments:
   ISQL allows applications to register the user-defined functions with the
   same name but varying text representations.  

See Also:
   ISQLFUNC_CreateFunction()
   ISQLFUNC_CreateAggregate()
   ISQLFUNC_CreateCollation()

================================================================================

Library load behavior flags

Description:
   The ISQLFUNC_LoadLibrary() call can be used to load a user-defined function
   library and reigster with with an ISQL object or manage the list of libraries
   that are automatically loaded with a particular database file. Applications 
   ay also use these inlines as shortcuts:

===pre>   
   ISQLFUNC_LoadLibraryByMimetype()
   ISQLFUNC_LoadLibraryByClassId() 
   ISQLFUNC_AddAutoLoadLibraryByMimetype()
   ISQLFUNC_AddAutoLoadLibraryByClassId()
   ISQLFUNC_RemoveAutoLoadLibraryByMimetype()
   ISQLFUNC_RemoveAutoLoadLibraryByClassId()
===/pre>
   
Definition:
===pre>
Definition                 Description
----------------------     --------------------------------------------------------
AEESQLF_REMOVEUDFLIBRARY   Remove the specified library from the table of libraries
                           that are automatically opened when the database file is
                           opened.

AEESQLF_LOADUDFLIBRARY     Load the specified library so the functions it 
                           implements can be called from this instance of ISQL.

AEESQLF_AUTOLOAUDFLIBRARY  Add the specified library to the table of libraries that 
                           are automatically opened when the database file is
                           opened.
===/pre>

Members:
   None
   
Comments:
   None

See Also:
   ISQLFUNC_LoadLibrary()
   ISQLFUNC_LoadLibraryByMimetype()
   ISQLFUNC_LoadLibraryByClassId() 
   ISQLFUNC_AddAutoLoadLibraryByMimetype()
   ISQLFUNC_AddAutoLoadLibraryByClassId()
   ISQLFUNC_RemoveAutoLoadLibraryByMimetype()
   ISQLFUNC_RemoveAutoLoadLibraryByClassId()

================================================================================

AEESQLFuncParam

Description:
   This structure is used to pass parameters in and out of user-defined 
   functions.  When a user-defined function is invoked, it is passed a list
   if AEESQLFuncParam arguments containing the values passed in from the
   SQL statement they were called from.  On completion, a user-defined function
   can return its results in a AEESQLFuncParam structure.

   Do not access the members of this structure directly.  Use the inlines
   described below to initialize and retrieve data from these structures.

Definition:
   typedef struct {
      int eType;                    // data type
      int dwSize;                   // size of data
      int dwFlags;                  // flags, like SQLF_WIDE
      void (*pfnFree)(void *pData); // data dtor, if required
      aeesql_uint64_struct i;       // integer
      const void *p;                // pointer data
   } AEESQLFuncParam;

Members:
===pre>
   eType:   The type of data stored in this AEESQLFuncParam
   dwSize:  The size of the data in this AEESQLFuncParam
   dwFlags: Flags used to interpret the data, such as SQLF_WIDE
   pfnFree: Callback to free the data if necessary.
   i:       An array of two uint32s used to pass integer data.
   p:       A void * for data
===/pre>

Comments:
   The following inlines are available to simplify extracting data from
   an AEESQLFuncParam structure.  They each take an AEESQLFuncParam, extract
   the data and possibly its length and return an error code.  These inlines
   are intended to be used by the  user-defined function implementations
   themselves to extract the paramters passed to them.

===pre>
   int GetAEESQLFuncParamInt(AEESQLFuncParam *p, int *pdwValue);
   int GetAEESQLFuncParamInt64(AEESQLFuncParam *p, int64 *piValue);
   int GetAEESQLFuncParamFloat(AEESQLFuncParam *p, double *pdValue);
   int GetAEESQLFuncParamText(AEESQLFuncParam *p, const char **ppszText, 
                              int *dwNumBytes);
   int GetAEESQLFuncParamWText(AEESQLFuncParam *p, const AECHAR **ppawText, 
                              int *dwNumBytes);
   int GetAEESQLFuncParamBlob(AEESQLFuncParam *p, const void **ppBlob, 
                              int *dwNumBytes);
===/pre>

   The following inlines are available to simplify setting data in an
   AEESQLFuncParam structure.  They each take an AEESQLFuncParam, a 
   value, an length in some cases and a callback function to be invoked
   to free the data if needed.  These inlines are intended to be used by
   user-defined function implementations to fill in the AEESQLFuncParam
   result parameter.

===pre>
   void SetAEESQLFuncParamInt(AEESQLFuncParam *p, int dwValue);
   void SetAEESQLFuncParamInt64(AEESQLFuncParam *p, int64 diValue);
   void SetAEESQLFuncParamFloat(AEESQLFuncParam *p, double dValue);
   void SetAEESQLFuncParamText(AEESQLFuncParam *p, const char *pszText, 
                               int dwNumBytes, void (*pfnFree)(void *pData));
   void SetAEESQLFuncParamWText(AEESQLFuncParam *p, const AECHAR *pawText, 
                                int dwNumBytes, void (*pfnFree)(void *pData));
   void SetAEESQLFuncParamBlob(AEESQLFuncParam *p, const void *pBlob, 
                                int dwNumBytes, void (*pfnFree)(void *pData))
   void SetAEESQLFuncParamNull(AEESQLFuncParam *p);
===/pre>

See Also:
   PFNSQLUDFFUNC
   PFNSQLUDFSTEP
   PFNSQLUDFFINAL

================================================================================

PFNSQLUDFFUNC

Description:
   ISQL supports simple functions that may be executed directly from SQL 
   queries.  These functions are called from statements, passed a number of 
   arguments and return a single result.  Functions may be used to perform
   some calculation or a specific task.
   
   Applications may implement such functions using this prototype, register 
   them with a call to ISQLFUNC_CreateFunction() then call them from SQL 
   queries.

Definition:
   typedef int (*PFNSQLUDFFUNC)(void *pUserData, 
                                int nArg,
                                AEESQLFuncParam *pArgs, 
                                AEESQLFuncParam *pResult);  

Members:
===pre>
   pUserData:  a pointer to the user data specifed by the entity that registered 
               the function with ISQLFUNC_CreateFunction().
   nArg:       the number of arguments passed to the function. That is, the number
               of AEESQLFuncParam structures in the pArgs array.  This value is
               specifed by the entity that registered the function with 
               ISQLFUNC_CreateFunction().
   pArgs:      an array of <nArg> AEESQLFuncParam structures
   pResult:    where the function should place the results.
===/pre>

Return Value:
   On success, SUCCESS must be returned. Otherwise an error code is returned.

Comments:
   A user-defined function might be executed from an SQL statement like this:

===pre>
   SELECT MyFunction('some string', 5);
===/pre>

See Also:
   ISQLFUNC_CreateFunction()
   ISQLFUNC_CreateWFunction()

================================================================================

PFNSQLUDFSTEP

Description:
   ISQL supports aggregate functions that are made up of a step function that's
   defined here and a finalize function.  The step function is called once per
   result, then the finalize function is called at the end.  Aggregates may be
   useful to perform some calculation on a range of values from a table.
   
   Applications may implement such functions using this prototype, register 
   them with a call to ISQLFUNC_CreateAggregate() then call them from SQL 
   queries.

Definition:
   typedef int (*PFNSQLUDFSTEP)(void *pUserData, 
                                int dwStep, 
                                int nArg, 
                                AEESQLFuncParam *pArgs, 
                                AEESQLFuncParam *pResult); 

Members:
===pre>
   pUserData:  a pointer to the user data specifed by the entity that registered 
               the function with ISQLFUNC_CreateFunction().
   dwStep:     step count, 1-based, increments per step call
   nArg:       the number of arguments passed to the function. That is, the number
               of AEESQLFuncParam structures in the pArgs array.  This value is
               specifed by the entity that registered the function with 
               ISQLFUNC_CreateFunction().
   pArgs:      an array of <nArg> AEESQLFuncParam structures
   pResult:    where the function should place the results
===/pre>

Return Value:
   On success, SUCCESS must be returned. Otherwise an error code is returned.

Comments:
   A user-defined aggregate might be executed from an SQL statement like this:

===pre>
   SELECT MyAggregate(id) from TestTable;
===/pre>

See Also:
   ISQLFUNC_CreateAggregate()
   ISQLFUNC_CreateWAggregate()
    
================================================================================

PFNSQLUDFFINAL

Description:
   ISQL supports aggregate functions that are made up of a step function and a 
   finalize function that's defined here.  The step function is called once per
   result, then the finalize function is called at the end.  Aggregates may be
   useful to perform some calculation on a range of values from a table.
   
   Applications may implement such functions using this prototype, register 
   them with a call to ISQLFUNC_CreateAggregate() then call them from SQL 
   queries.

Definition:
   typedef int (*PFNSQLUDFFINAL)(void *pUserData, AEESQLFuncParam *pResult);  

Members:
===pre>
   pUserData:  a pointer to the user data specifed by the entity that registered 
               the function with ISQLFUNC_CreateFunction().
   pResult:    where the function should place the results.
===/pre>

Return Value:
   On success, SUCCESS must be returned. Otherwise an error code is returned.

Comments:
   A user-defined aggregate might be executed from an SQL statement like this:

===pre>
   SELECT MyAggregate(id) from TestTable;
===/pre>

See Also:
   ISQLFUNC_CreateAggregate()
   ISQLFUNC_CreateWAggregate()

================================================================================

PFNSQLUDFCOMPARE

Description:
   ISQL supports user-defined collations, or compare functions.  Applications 
   may implement their own collations to sort strings a certain way or provide
   more complex compare functions for complex languages.

   Applications may implement such functions using this prototype, register 
   them with a call to ISQLFUNC_CreateCollation() then call them from SQL 
   queries.

Definition:
    typedef int (*PFNSQLUDFCOMPARE)(void *pUserData, 
                                    int dwLen1, 
                                    const void *pString1, 
                                    int dwLen2, 
                                    const void *pString2);

Members:
===pre>
   pUserData: a pointer to the user data specifed by the entity that registered 
              the function with ISQLFUNC_CreateFunction().
   dwLen1:    the length of the first string to compare
   pString:   the first string to compare
   dwLen2:    the length of the second string to compare
   pString2:  the second string to compare
===/pre>

Return Value:
   (-1) if pString1 is "less than" pString2
    (0) if the strings are equal
    (1) if pString1 is "greater than" pString2

Comments:
   A user-defined collation might be executed from an SQL statement like this:

===pre>
   SELECT * FROM TestTable ORDER BY name COLLATE MyCompare;
===/pre>

See Also:
   ISQLFUNC_CreateCollation()
   ISQLFUNC_CreateWCollation()  

================================================================================
 INTERFACE DOCUMENTATION
================================================================================

ISQLFunc Interface

Description: 
   The ISQLFunc interface provides applications with methods to register user-
   defined functions, aggregates and collations with a particular ISQL object.
   It also provides a way for applications to associate libraries of user-
   defined functions with a particular database so that they're automatically
   loaded when the database is opened.

   User-defined functions may have single or double byte names, are implemented
   either within an application or a user-defined function library and are 
   registered using ISQLFUNC_CreateFunction() or ISQLFUNC_CreateWFunction().

   User-defined aggregate functions are made up of a step function and a 
   finalize function that are called on a range of values.  The are registered
   using ISQLFUNC_CreateAggregate() or ISQLFUNC_CreateWAggregate().

   User-defined collations are string sort implementations that a developer may
   create to perform custom sorting for query results.  They are registered
   using ISQLFUNC_CreateCollation() or ISQLFUNC_CreateWCollation().

   To unregister a function, aggregate or collation, any of the above 
   methods can be called with the name of the function to unregister and a
   NULL function pointer.
   
   The ISQLFunc interface allows applications to register functions with a
   particular ISQL object.  ISQLFunc is a derived class of ISQL.  Registering
   a function using a ISQLFunc method registers it with the ISQL object the
   ISQLFunc instance was retrieved from.  Once a function has been registered,
   it is available for use by the ISQL object that it registered with.  Any 
   other ISQL objects on the system do not have access to the registered
   functions.

===H2>
User-defined function libraries
===/H2>

   User-defined functions may also be implemeted within BREW extensions that
   adhere to the ISQLUDFLib interface.  These are standard BREW extensions 
   that implement user-defined function, aggregates and/or collations,
   expose an ISQLUDFLib interface by class id or mimetype and use ISQLFunc
   to register its functions. ISQLFunc provides a method to load user-defined
   function libraries via their class id or mimetype.  No support is provided
   to unload a user-defined function library.  These are unloaded once the
   ISQL object they're registered to is released.

   ISQLFunc provides the ability to associate user-defined function libraries
   with a database file via a special table.  This table contains the class
   id or mimetype of ISQLUDFLib objects.  When the database is opened and
   the SQLF_LOAD_EXTENSIONS flag is set in the call to ISQLMGR_Open(), the
   table is read and each extension is loaded and registered.

===H2>
Required header files
===/H2>
===pre>
AEESQLFunc.h
===/pre>

See Also:
   ISQL
   ISQLUDFLib

================================================================================

ISQLFUNC_AddRef() 

Description: 
   This function increments the reference count of the ISQLFunc interface 
   object.  This allows the object to be shared by multiple callers. The object 
   is freed when the reference count reaches 0 (zero).

Prototype:
   uint32 ISQLFUNC_AddRef(ISQLFunc *pISQLFunc);

Parameters:
===pre>
   pISQLFunc: Pointer to the ISQLFunc Interface object
===/pre> 

Return Value:
   Incremented reference count for the object. 

Comments: 
   A valid object returns a positive reference count.

See Also:
   ISQLFUNC_Release() 

================================================================================

ISQLFUNC_QueryInterface()

Description:
   This function queries an interface for an alterrnate interface that may 
   provide extended functionality or a proprietary extension.  If successful, 
   the function will return a pointer to the requested interface in the memory 
   location pointed to by the 'd' parameter.  The requested interface is 
   identified by a globally unique interface ID passed to the function in the 
   'c' parameter.

Prototype:
   int ISQLFUNC_QueryInterface(ISQLFunc *p, AEECLSID c, void **d);

Parameters:
===pre>
   p:  Pointer to the ISQLFunc interface object.
   c:  A globally unique id that identifies the interface to be queried.
   d:  Pointer to a pointer returned by the function to reference the
       interface requested by the 'c' parameter.  If the queried item is not
       available, this pointer is returned as NULL.
===/pre> 

Return Value:
===pre>
   SUCCESS          - The requested interface was found and has been returned
                      in the 'd' parameter.
   ECLASSNOTSUPPORT - The requested interface is not supported.
===/pre> 

Comments:
   The pointer passed in the 'd' parameter is a double-indirection -- i.e. it 
   is a pointer to a pointer.  Applications should use care in ensuring that the 
   proper level of indirection is used in calling the function.

Side Effects:
   Upon successful retrieval of an interface pointer, that interface is returned 
   with its reference count incremented.  The caller is responsible for 
   releasing this object at some point in the future.

See Also:
   IQI_QueryInterface()

================================================================================

ISQLFUNC_Release() 

Description: 
   This function decrements the reference count of the ISQLFunc Interface object. 
   The object is freed from memory and is no longer valid once the reference 
   count reaches 0 (zero). 

Prototype:
   uint32 ISQLFUNC_Release(ISQLFunc *pISQLFunc);
    
Parameters:
===pre>
   pISQLFunc: Pointer to the ISQLFunc Interface object
===/pre> 

Return Value:
===pre> 
   The decremented reference count for the object.
   0 (zero) if the object has been freed and is no longer valid.
===/pre> 

See Also:
   ISQLFUNC_AddRef() 

================================================================================

ISQLFUNC_LoadLibrary()

Description: 
   This function loads the user-defined function library identified by a
   mimetype if specified and a classid otherwise or manages the auto-load table
   in the current database file.  The auto-load table is named according to the
   AEESQL_UDFAUTOLOADTABLENAME constant once it's created.

   If a mimetype is specified, ISHELL_GetHandler() is called with a handler of
   AEEIID_SQLUDFLIB on the mimetype to discover the library to be loaded.  
   Otherwise, the class id passed in is used to find the library to be loaded.

   The dwFlags parameter tells this function to either load the library, add
   it to the table of auto-load libraries or remove it from that table.

   Applications won't typically call this routine.  Instead, they should call
   one of the following:

===pre>
   ISQLFUNC_LoadLibraryByMimetype()
   ISQLFUNC_LoadLibraryByClassId() 
   ISQLFUNC_AddAutoLoadLibraryByMimetype()
   ISQLFUNC_AddAutoLoadLibraryByClassId()
   ISQLFUNC_RemoveAutoLoadLibraryByMimetype()
   ISQLFUNC_RemoveAutoLoadLibraryByClassId()
===/pre> 

Prototype:
   int ISQLFUNC_LoadLibrary(ISQLFunc *piISQLFunc, const char *pszMimeType, AEECLSID clsId, int dwFlags);
    
Parameters:
===pre>
   pISQLFunc:     Pointer to the ISQLFunc Interface object
   pszMimeType:   Mimetype to use to discover the library to load
   clsId:         ClassId of the library to load
   dwFlags:       Flags that govern what this routine does.
===/pre> 

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

See Also:
   Library load behavior flags
   ISQLFUNC_LoadLibraryByMimetype()
   ISQLFUNC_LoadLibraryByClassId() 
   ISQLFUNC_AddAutoLoadLibraryByMimetype()
   ISQLFUNC_AddAutoLoadLibraryByClassId()
   ISQLFUNC_RemoveAutoLoadLibraryByMimetype()
   ISQLFUNC_RemoveAutoLoadLibraryByClassId()

================================================================================

ISQLFUNC_LoadLibraryByMimetype()

Description: 
   This function calls ISHELL_GetHandler() with a handler of AEEIID_SQLUDFLIB
   to determine the class id of the library that will handle the specified
   mimetype.  The library is then loaded and intialized, and its functions
   are registered with the ISQL isntance the piISQLFunc object was retrieved
   from.

Prototype:
   int ISQLFUNC_LoadLibraryByMimetype(ISQLFunc *piISQLFunc, 
                                      const char *pszMimeType);
    
Parameters:
===pre>
   pISQLFunc:   Pointer to the ISQLFunc Interface object
   pszMimeType: Mimetype to use to discover the library to load
===/pre> 

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

Comments:
   This function is equivalent to calling ISQLFUNC_LoadLibrary() with the
   AEESQLF_LOADUDFLIBRARY flag set.

See Also:
   ISQLFUNC_LoadLibraryByClassId()
   ISQLFUNC_LoadLibrary()

================================================================================

ISQLFUNC_LoadLibraryByClassId()

Description: 
   This function loads the user-defined function library specified by the
   class id and registers its functions with the ISQL instance the piISQLFunc 
   object was retrieved from.

Prototype:
   int ISQLFUNC_LoadLibraryByClassId(ISQLFunc *piISQLFunc, 
                                     AEECLSID clsId);
    
Parameters:
===pre>
   pISQLFunc: Pointer to the ISQLFunc Interface object
   clsId:     ClassId of the library to load
===/pre> 

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

Comments:
   This function is equivalent to calling ISQLFUNC_LoadLibrary() with the
   AEESQLF_LOADUDFLIBRARY flag set.

See Also:
   ISQLFUNC_LoadLibraryByMimetype() 
   ISQLFUNC_LoadLibrary()

================================================================================
 
ISQLFUNC_AddAutoLoadLibraryByMimetype()

Description: 
   This function adds the specified mimetype to the library auto-load table in
   the database file the piISQLFunc object currently has open.  Next time the 
   database file is opened with ISQLMGR_Open() with the SQLF_LOAD_EXTENSIONS
   flag set, ISQL will use ISHELL_GetHandler() with a handler of 
   AEEIID_SQLUDFLIB to automatically discover and load the library.

Prototype:
   int ISQLFUNC_AddAutoLoadLibraryByMimetype(ISQLFunc *piISQLFunc, 
                                             const char *pszMimeType);
    
Parameters:
===pre>
   pISQLFunc:   Pointer to the ISQLFunc Interface object
   pszMimeType: Mimetype of the library to add to the auto-load table
===/pre> 

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

Comments:
   This function is equivalent to calling ISQLFUNC_LoadLibrary() with the
   AEESQLF_AUTOLOAUDFLIBRARY flag set.

See Also:
   ISQLFUNC_AddAutoLoadLibraryByClassId() 
   ISQLFUNC_LoadLibrary()

================================================================================

ISQLFUNC_AddAutoLoadLibraryByClassId()

Description: 
   This function adds the specified class id to the library auto-load table in
   the database file the piISQLFunc object currently has open.  Next time the 
   database file is opened with ISQLMGR_Open() with the SQLF_LOAD_EXTENSIONS
   flag set, ISQL will automatically load the library.

Prototype:
   int ISQLFUNC_AddAutoLoadLibraryByClassId(ISQLFunc *piISQLFunc, 
                                            AEECLSID clsId);
    
Parameters:
===pre>
   pISQLFunc: Pointer to the ISQLFunc Interface object
   clsId:     Class id of the library to add to the auto-load table
===/pre> 

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

Comments:
   This function is equivalent to calling ISQLFUNC_LoadLibrary() with the
   AEESQLF_AUTOLOAUDFLIBRARY flag set.

See Also:
   ISQLFUNC_AddAutoLoadLibraryByMimetype()
   ISQLFUNC_LoadLibrary()

================================================================================

ISQLFUNC_RemoveAutoLoadLibraryByMimetype()

Description: 
   This function removes the specified mimetype from the library auto-load 
   table in the database file the piISQLFunc object currently has open.  
   Applications may use this routine to disassociate a library with a database
   file.

Prototype:
   int ISQLFUNC_RemoveAutoLoadLibraryByMimetype(ISQLFunc *piISQLFunc, 
                                                const char *pszMimeType);
    
Parameters:
===pre>
   pISQLFunc:   Pointer to the ISQLFunc Interface object
   pszMimeType: Mimetype of the library to remove to the auto-load table
===/pre> 

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

Comments:
   This function is equivalent to calling ISQLFUNC_LoadLibrary() with the
   AEESQLF_REMOVEUDFLIBRARY flag set.

See Also:
   ISQLFUNC_RemoveAutoLoadLibraryByClassId() 
   ISQLFUNC_LoadLibrary()

================================================================================

ISQLFUNC_RemoveAutoLoadLibraryByClassId()

Description: 
   This function removes the specified class id from the library auto-load 
   table in the database file the piISQLFunc object currently has open.  
   Applications may use this routine to disassociate a library with a database
   file.

Prototype:
   int ISQLFUNC_RemoveAutoLoadLibraryByClassId(ISQLFunc *piISQLFunc, 
                                               AEECLSID clsId);
    
Parameters:
===pre>
   pISQLFunc: Pointer to the ISQLFunc Interface object
   clsId:     Class id of the library to remove to the auto-load table
===/pre> 

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

Comments:
   This function is equivalent to calling ISQLFUNC_LoadLibrary() with the
   AEESQLF_REMOVEUDFLIBRARY flag set.

See Also:
   ISQLFUNC_RemoveAutoLoadLibraryByMimetype()
   ISQLFUNC_LoadLibrary()

================================================================================

ISQLFUNC_CreateFunction()

Description: 
   Applications or user-defined function librariess call this routine to 
   register an implementation of a user-defined function with a particular
   database connection.

Prototype:
   int ISQLFUNC_CreateFunction(ISQLFunc *piISQLFunc, 
                               const char *pFuncName, 
                               int nArgs, 
                               int dwTextRep, 
                               PFNSQLUDFFUNC pfnFunc, 
                               void *pUserData);
    
Parameters:
===pre>
   piISQLFunc: a pointer to the ISQLFunc interface
   pFuncName:  the function name. This is how the function will be called
               from SQL statements.
   nArgs:      the number of arguments the user-defined function takes
   dwTextRep:  the text representation of the parameters (AEESQL_UTF8, 
               AEESQL_UTF16, etc.)
   pfnFunc:    a pointer to the user-defined function implemented by the 
               caller
   pUserData:  a pointer to user data that will be passed into pfnFunc
===/pre> 

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

Comments:
   A function registered with this routine is only availabe to be called from
   the ISQL/ISQLFunc object it registered with.

   To unregister the function created with this routine, make the exact call
   again but set pfnFunc to NULL.

See Also:
   ISQLFUNC_CreateWFunction() 

================================================================================

ISQLFUNC_CreateWFunction()

Description: 
   Applications or user-defined function librariess call this routine to 
   register an implementation of a user-defined function with a particular
   database connection.

Prototype:
   int ISQLFUNC_CreateWFunction(ISQLFunc *piISQLFunc, 
                                const AECHAR *pawFuncName, 
                                int nArgs, 
                                int dwTextRep, 
                                PFNSQLUDFFUNC pfnFunc, 
                                void *pUserData);
    
Parameters:
===pre>
   piISQLFunc:  a pointer to the ISQLFunc interface
   pawFuncName: the function name. This is how the function will be called
                from SQL statements.
   nArgs:       the number of arguments the user-defined function takes
   dwTextRep:   the text representation of the parameters (AEESQL_UTF8, 
                AEESQL_UTF16, etc.)
   pfnFunc:     a pointer to the user-defined function implemented by the 
                caller
   pUserData:   a pointer to user data that will be passed into pfnFunc
===/pre> 

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

Comments:
   A function registered with this routine is only availabe to be called from
   the ISQL/ISQLFunc object it registered with.

   To unregister the function created with this routine, make the exact call
   again but set pfnFunc to NULL.

See Also:
   ISQLFUNC_CreateFunction() 

================================================================================

ISQLFUNC_CreateAggregate()

Description: 
   Applications or user-defined function librariess call this routine to 
   register an implementation of a user-defined aggregate function with a 
   particular database connection. 

Prototype:
   int ISQLFUNC_CreateAggregate(ISQLFunc *piISQLFunc, 
                                const char *pFuncName, 
                                int nArgs, 
                                int dwTextRep, 
                                PFNSQLUDFSTEP pfnStep, 
                                PFNSQLUDFFINAL pfnFinal, 
                                void *pUserData);
    
Parameters:
===pre>
   piISQLFunc: a pointer to the ISQLFunc interface
   pFuncName:  the function name. This is how the function will be called
               from SQL statements.
   nArgs:      the number of arguments the user-defined function takes
   dwTextRep:  the text representation of the parameters (AEESQL_UTF8, 
               AEESQL_UTF16, etc.)
   pfnStep:    a pointer to the user-defined step function implemented by 
               the caller
   pfnFinal:   a pointer to the user-defined finalize function implemented 
               by the caller
   pUserData:  a pointer to user data that will be passed into pfnFunc
===/pre> 

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

Comments:
   A function registered with this routine is only availabe to be called from
   the ISQL/ISQLFunc object it registered with.

   To unregister the function created with this routine, make the exact call
   again but set pfnStep and pfnFinal to NULL.

See Also:
   ISQLFUNC_CreateWAggregate() 

================================================================================

ISQLFUNC_CreateWAggregate()

Description: 
   Applications or user-defined function librariess call this routine to 
   register an implementation of a user-defined aggregate function with a 
   particular database connection. 

Prototype:
   int ISQLFUNC_CreateWAggregate(ISQLFunc *piISQLFunc, 
                                 const AECHAR *pawFuncName, 
                                 int nArgs, 
                                 int dwTextRep, 
                                 PFNSQLUDFSTEP pfnStep, 
                                 PFNSQLUDFFINAL pfnFinal, 
                                 void *pUserData);
    
Parameters:
===pre>
   piISQLFunc:  a pointer to the ISQLFunc interface
   pawFuncName: the function name. This is how the function will be called
                from SQL statements.
   nArgs:       the number of arguments the user-defined function takes
   dwTextRep:   the text representation of the parameters (AEESQL_UTF8,
                AEESQL_UTF16, etc.)
   pfnStep:     a pointer to the user-defined step function implemented by 
                the caller
   pfnFinal:    a pointer to the user-defined finalize function implemented by 
                the caller
   pUserData:   a pointer to user data that will be passed into pfnFunc
===/pre> 

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

Comments:
   A function registered with this routine is only availabe to be called from
   the ISQL/ISQLFunc object it registered with.

   To unregister the function created with this routine, make the exact call
   again but set pfnStep and pfnFinal to NULL.

See Also:
   ISQLFUNC_CreateAggregate() 

================================================================================

ISQLFUNC_CreateCollation()

Description: 
   Applications or user-defined function librariess call this routine to 
   register an implementation of a user-defined collation with a particular 
   database connection. 

Prototype:
   int ISQLFUNC_CreateCollation(ISQLFunc *piISQLFunc, 
                                const char *pFuncName, 
                                int dwTextRep, 
                                PFNSQLUDFCOMPARE pfnCompare, 
                                void *pUserData);
    
Parameters:
===pre>
   piISQLFunc: a pointer to the ISQLFunc interface
   pFuncName:  the function name. This is how the function will be called 
               from SQL statements.
   dwTextRep:  the text representation of the parameters (AEESQL_UTF8, 
               AEESQL_UTF16, etc.)
   pfnCompare: a pointer to the user-defined compare function implemented by 
               the caller
   pUserData:  a pointer to user data that will be passed into pfnFunc
===/pre> 

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

Comments:
   A function registered with this routine is only availabe to be called from
   the ISQL/ISQLFunc object it registered with.

   To unregister the function created with this routine, make the exact call
   again but set pfnCompare to NULL.

See Also:
   ISQLFUNC_CreateWCollation() 

================================================================================

ISQLFUNC_CreateWCollation() 

Description: 
   Applications or user-defined function librariess call this routine to 
   register an implementation of a user-defined collation with a particular 
   database connection. 

Prototype:
   ISQLFUNC_CreateWCollation(ISQLFunc *piISQLFunc, 
                             const AECHAR *pawFuncName, 
                             int dwTextRep, 
                             PFNSQLUDFCOMPARE pfnCompare, 
                             void *pUserData);
    
Parameters:
===pre>
   piISQLFunc:  a pointer to the ISQLFunc interface
   pawFuncName: the function name. This is how the function will be called 
                from SQL statements.
   dwTextRep:   the text representation of the parameters (AEESQL_UTF8, 
                AEESQL_UTF16, etc.)
   pfnCompare:  a pointer to the user-defined compare function implemented by 
                the caller
   pUserData:   a pointer to user data that will be passed into pfnFunc
===/pre> 

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

Comments:
   A function registered with this routine is only availabe to be called from
   the ISQL/ISQLFunc object it registered with.

   To unregister the function created with this routine, make the exact call
   again but set pfnCompare to NULL.

See Also:
   ISQLFUNC_CreateCollation() 

================================================================================
  DATA STRUCTURE DOCUMENTATION
================================================================================
ISQLUDFLib Interface and Class IDs

Description:
   Each class and interface is identified by a unique ID.  These IDs are used to 
   distinguish between various types of objects.

   ISQLUDFLib supports both an interface and the creation of class object 
   instances. The ISQLUDFLib interface or a new instance of the ISQLUDFLib 
   class object may be created by calling ISHELL_CreateInstance() and passing in 
   the appropriate class id.

   ISQLUtil recognizes the followinginterface IDs.  Class ids will be defined
   by the individual implementations of ISQLUDFLib.

Definition:

Identifier           Definition
----------           -----------------------------------------------------------
AEEIID_SQLUDFLIB:    Identifies the ISQLUDFLib interface

Comments:
   None

See Also:
   ISHELL_CreateInstance()

================================================================================
 INTERFACE DOCUMENTATION
================================================================================

ISQLUDFLib Interface

Description: 
  The ISQLUDFLib interface provides developers with the ability to create
  libraries of SQL user-defiend functions, aggregates and collations and 
  package them as standard BREW extensions.  

  A developer that wishes to create a library of user-defined functions for
  applications that use ISQL would create a standard BREW extension that
  implements an instance of ISQLUDFLib.  The ISQLUDFLib implementation would
  include the user-defined functions themselves along with calls to 
  ISQLFUNC_CreateFunction() and related to register the individual functions
  from its initialize method.

  User-defined function libraries can advertise themselves using a BREW class
  id generated using the BREW ClassID generator.  
  
  Libraries may also identify themselves via a mimetype.  The "x-qc-sqludflib"
  mimetype (AEESQL_UDFLIBMIMETYPE) has been defined for this purpose.  Libraries
  may declare themselves as AEESQL_UDFLIBMIMETYPE mimetype handlers 
  "x-qc-sqludflib/mylib" and be discovered by calling ISHELL_GetHandler() with
  AEEIID_SQLUDFLIB as the handler.

  Once a user-defined function library has been registered with an instance
  of ISQL, the application may release its pointer to it.  The ISQL object
  will track the library and release it when it's released. 

===H2>
Required header files
===/H2>
===pre>
AEESQLFunc.h
===/pre>

See Also:
   ISQLFunc

================================================================================

ISQLUDFLIB_AddRef() 

Description: 
   This function increments the reference count of the ISQLUDFLib interface 
   object.  This allows the object to be shared by multiple callers. The object 
   is freed when the reference count reaches 0 (zero).

Prototype:
   uint32 ISQLUDFLIB_AddRef(ISQLUDFLib *pISQLUDFLib);

Parameters:
===pre>
   pISQLUDFLib: Pointer to the ISQLUDFLib Interface object
===/pre> 

Return Value:
   Incremented reference count for the object. 

Comments: 
   A valid object returns a positive reference count.

See Also:
   ISQLUDFLIB_Release() 

================================================================================

ISQLUDFLIB_Init() 

Description: 
   This is the entry point for a user-defined function library.  A library is
   created via class id (itself perhaps discovered by mimetype) using 
   ISHELL_CreateInstance().  The ISQLUDFLIB_Init() routine is then called to 
   intialize the library.  The library implementation can allocate any memory
   or interfaces it needs in this routine, then register the functions it
   implements with the piSQLFunc passed in.

Prototype:
   int ISQLUDFLIB_Init(ISQLUDFLib *po, ISQLFunc *piSQLFunc);

Parameters:
===pre>
   pISQLUDFLib: Pointer to the ISQLUDFLib Interface object
   piSQLFunc:   Pointer to the ISQLFunc object the library should use to
                register its functions with.
===/pre> 

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

Comments: 
   None

See Also:
   ISQLUDFLIB_Release() 
   ISQLFUNC_CreateFunction()
   ISQLFUNC_CreateWFunction()
   ISQLFUNC_CreateAggregate()
   ISQLFUNC_CreateWAggregate()
   ISQLFUNC_CreateCollation()
   ISQLFUNC_CreateWCollation()  

================================================================================

ISQLUDFLIB_QueryInterface()

Description:
   This function queries an interface for an alterrnate interface that may 
   provide extended functionality or a proprietary extension.  If successful, 
   the function will return a pointer to the requested interface in the memory 
   location pointed to by the 'd' parameter.  The requested interface is 
   identified by a globally unique interface ID passed to the function in the 
   'c' parameter.

Prototype:
   int ISQLUDFLIB_QueryInterface(ISQLUDFLib *p, AEECLSID c, void **d);

Parameters:
===pre>
   p:  Pointer to the ISQLUDFLib interface object.
   c:  A globally unique id that identifies the interface to be queried.
   d:  Pointer to a pointer returned by the function to reference the
       interface requested by the 'c' parameter.  If the queried item is not
       available, this pointer is returned as NULL.
===/pre> 

Return Value:
===pre>
   SUCCESS          - The requested interface was found and has been returned
                      in the 'd' parameter.
   ECLASSNOTSUPPORT - The requested interface is not supported.
===/pre> 

Comments:
   The pointer passed in the 'd' parameter is a double-indirection -- i.e. it 
   is a pointer to a pointer.  Applications should use care in ensuring that the 
   proper level of indirection is used in calling the function.

Side Effects:
   Upon successful retrieval of an interface pointer, that interface is returned 
   with its reference count incremented.  The caller is responsible for 
   releasing this object at some point in the future.

See Also:
   IQI_QueryInterface()

================================================================================

ISQLUDFLIB_Release() 

Description: 
   This function decrements the reference count of the ISQLUDFLib Interface 
   object. The object is freed from memory and is no longer valid once the 
   reference count reaches 0 (zero). 

   User-defined libraries would release any interfaces of memory allocated in
   support of the functions the implement in this routine.

Prototype:
   uint32 ISQLUDFLIB_Release(ISQLUDFLib *pISQLUDFLib);
    
Parameters:
===pre>
   pISQLUDFLib: Pointer to the ISQLUDFLib Interface object
===/pre> 

Return Value:
===pre> 
   The decremented reference count for the object.
   0 (zero) if the object has been freed and is no longer valid.
===/pre> 

See Also:
   ISQLUDFLIB_AddRef() 

==============================================================================*/


#endif   // __AEEISQLFUNC_H__
